@@ -230,9 +230,10 @@ public class I2PTunnelHTTPServer extends I2PTunnelServer {
                 long pb = 1000L * getIntOption(OPT_POST_BAN_TIME, DEFAULT_POST_BAN_TIME);
                 long px = 1000L * getIntOption(OPT_POST_TOTAL_BAN_TIME, DEFAULT_POST_TOTAL_BAN_TIME);
                 if (_postThrottler == null)
-                    _postThrottler = new ConnThrottler(pp, pt, pw, pb, px, "POST", _log);
+                    _postThrottler = new ConnThrottler(pp, pt, pw, pb, px, "POST/PUT", _log);
                 else
                     _postThrottler.updateLimits(pp, pt, pw, pb, px);
+                _postThrottler.start();
             }
         }
     }
@@ -254,7 +255,7 @@ public class I2PTunnelHTTPServer extends I2PTunnelServer {
     public boolean close(boolean forced) {
         synchronized(this) {
             if (_postThrottler != null)
-                _postThrottler.clear();
+                _postThrottler.stop();
         }
         return super.close(forced);
     }
@@ -481,10 +482,11 @@ public class I2PTunnelHTTPServer extends I2PTunnelServer {
 
             if (_postThrottler != null &&
                 command.length() >= 5 &&
-                command.substring(0, 5).toUpperCase(Locale.US).equals("POST ")) {
+                (command.substring(0, 5).toUpperCase(Locale.US).equals("POST ") ||
+                 command.substring(0, 4).toUpperCase(Locale.US).equals("PUT "))) {
                 if (_postThrottler.shouldThrottle(peerHash)) {
                     if (_log.shouldLog(Log.WARN))
-                        _log.warn("Refusing POST since peer is throttled: " + peerB32);
+                        _log.warn("Refusing POST/PUT since peer is throttled: " + peerB32);
                     try {
                         // Send a 429, so the user doesn't get an HTTP Proxy error message
                         // and blame his router or the network.
@@ -515,7 +517,12 @@ public class I2PTunnelHTTPServer extends I2PTunnelServer {
             }
             if (spoofHost != null)
                 setEntry(headers, "Host", spoofHost);
-            setEntry(headers, "Connection", "close");
+
+            // Force Connection: close, unless websocket
+            String conn = getEntryOrNull(headers, "Connection");
+            if (conn == null || !conn.toLowerCase(Locale.US).contains("upgrade"))
+                setEntry(headers, "Connection", "close");
+
             // we keep the enc sent by the browser before clobbering it, since it may have 
             // been x-i2p-gzip
             String enc = getEntryOrNull(headers, "Accept-Encoding");
@@ -641,7 +648,8 @@ public class I2PTunnelHTTPServer extends I2PTunnelServer {
                     _log.info("request headers: " + _headers);
                 serverout.write(DataHelper.getUTF8(_headers));
                 browserin = _browser.getInputStream();
-                // Don't spin off a thread for this except for POSTs
+                // Don't spin off a thread for this except for POSTs and PUTs
+                // TODO Upgrade:
                 // beware interference with Shoutcast, etc.?
                 if ((!(_headers.startsWith("GET ") || _headers.startsWith("HEAD "))) ||
                     browserin.available() > 0) {  // just in case
@@ -1042,6 +1050,8 @@ public class I2PTunnelHTTPServer extends I2PTunnelServer {
                     name = "User-Agent";
                 else if ("referer".equals(lcName))
                     name = "Referer";
+                else if ("connection".equals(lcName))
+                    name = "Connection";
 
                 // For incoming, we remove certain headers to prevent spoofing.
                 // For outgoing, we remove certain headers to improve anonymity.
