@@ -66,7 +66,7 @@ public class TunnelController implements Logging {
         DESTROYING,
         DESTROYED,
     }
-    
+
     public static final String KEY_BACKUP_DIR = "i2ptunnel-keyBackup";
 
     /** all of these @since 0.9.14 */
@@ -85,7 +85,8 @@ public class TunnelController implements Logging {
     public static final String PROP_TARGET_HOST = "targetHost";
     public static final String PROP_TARGET_PORT = "targetPort";
     public static final String PROP_TYPE = "type";
-    
+    public static final String PROP_FILTER = "filterDefinition";
+
     /**
      * all of these are @since 0.9.33 (moved from TunnelConfig)
      */
@@ -97,7 +98,7 @@ public class TunnelController implements Logging {
     public static final String PROP_MAX_TOTAL_CONNS_DAY = "i2p.streaming.maxTotalConnsPerDay";
     public static final String PROP_MAX_STREAMS = "i2p.streaming.maxConcurrentStreams";
     public static final String PROP_LIMITS_SET = "i2p.streaming.limitsManuallySet";
-    public static final int DEFAULT_MAX_CONNS_MIN = 10;
+    public static final int DEFAULT_MAX_CONNS_MIN = 15;
     public static final int DEFAULT_MAX_CONNS_HOUR = 40;
     public static final int DEFAULT_MAX_CONNS_DAY = 100;
     public static final int DEFAULT_MAX_TOTAL_CONNS_MIN = 25;
@@ -207,7 +208,7 @@ public class TunnelController implements Logging {
         }
         _state = keyOK && getStartOnLoad() ? TunnelState.START_ON_LOAD : TunnelState.STOPPED;
     }
-    
+
     /**
      * @return success
      */
@@ -270,7 +271,7 @@ public class TunnelController implements Logging {
         }
         return true;
     }
-    
+
     /**
      * Creates alternate Destination with the same encryption keys as the primary Destination,
      * but a different signing key.
@@ -365,7 +366,7 @@ public class TunnelController implements Logging {
             if (out != null) try { out.close(); } catch (IOException ioe) {}
         }
     }
-    
+
     public void startTunnelBackground() {
         synchronized (this) {
             if (_state != TunnelState.STOPPED && _state != TunnelState.START_ON_LOAD)
@@ -373,7 +374,7 @@ public class TunnelController implements Logging {
         }
         new I2PAppThread(new Runnable() { public void run() { startTunnel(); } }, "Tunnel Starter " + getName()).start();
     }
-    
+
     /**
      * Start up the tunnel (if it isn't already running)
      *
@@ -466,7 +467,7 @@ public class TunnelController implements Logging {
         acquire();
         changeState(TunnelState.RUNNING);
     }
-    
+
     private void startHttpClient() {
         setListenOn();
         String listenPort = getListenPort();
@@ -477,7 +478,7 @@ public class TunnelController implements Logging {
         else
             _tunnel.runHttpClient(new String[] { listenPort, sharedClient, proxyList }, this);
     }
-    
+
     private void startConnectClient() {
         setListenOn();
         String listenPort = getListenPort();
@@ -488,7 +489,7 @@ public class TunnelController implements Logging {
         else
             _tunnel.runConnectClient(new String[] { listenPort, sharedClient, proxyList }, this);
     }
-    
+
     private void startIrcClient() {
         setListenOn();
         String listenPort = getListenPort();
@@ -501,7 +502,7 @@ public class TunnelController implements Logging {
             _tunnel.runIrcClient(new String[] { listenPort, dest, sharedClient }, this);
         }
     }
-    
+
     private void startSocksClient() {
         setListenOn();
         String listenPort = getListenPort();
@@ -520,7 +521,7 @@ public class TunnelController implements Logging {
             _tunnel.runSOCKSTunnel(new String[] { listenPort, sharedClient }, this);
         }
     }
-    
+
     /** @since 0.7.12 */
     private void startSocksIRCClient() {
         setListenOn();
@@ -540,7 +541,7 @@ public class TunnelController implements Logging {
             _tunnel.runSOCKSIRCTunnel(new String[] { listenPort, sharedClient }, this);
         }
     }
-    
+
     /*
      *  Streamr client is a UDP server, use the listenPort field for targetPort
      */
@@ -550,7 +551,7 @@ public class TunnelController implements Logging {
         String dest = getTargetDestination();
         _tunnel.runStreamrClient(new String[] { targetHost, targetPort, dest }, this);
     }
-    
+
     /**
      *  Streamr server is a UDP client, use the targetPort field for listenPort
      */
@@ -563,7 +564,7 @@ public class TunnelController implements Logging {
         String privKeyFile = getPrivKeyFile(); 
         _tunnel.runStreamrServer(new String[] { listenPort, privKeyFile }, this);
     }
-    
+
     /** 
      * Note the fact that we are using some sessions, so that they dont get
      * closed by some other tunnels
@@ -585,7 +586,7 @@ public class TunnelController implements Logging {
                 _log.warn("No sessions to acquire? for " + getName());
         }
     }
-    
+
     /** 
      * Note the fact that we are no longer using some sessions, and if
      * no other tunnels are using them, close them.
@@ -621,7 +622,7 @@ public class TunnelController implements Logging {
             sessions.addAll(_sessions);
         return sessions;
     }
-    
+
     private void startClient() {
         setListenOn();
         String listenPort = getListenPort(); 
@@ -641,7 +642,7 @@ public class TunnelController implements Logging {
         String privKeyFile = getPrivKeyFile(); 
         _tunnel.runServer(new String[] { targetHost, targetPort, privKeyFile }, this);
     }
-    
+
     private void startHttpServer() {
         String targetHost = getTargetHost(); 
         String targetPort = getTargetPort(); 
@@ -649,7 +650,7 @@ public class TunnelController implements Logging {
         String privKeyFile = getPrivKeyFile(); 
         _tunnel.runHttpServer(new String[] { targetHost, targetPort, spoofedHost, privKeyFile }, this);
     }
-    
+
     private void startHttpBidirServer() {
         setListenOn();
         String targetHost = getTargetHost();
@@ -666,14 +667,14 @@ public class TunnelController implements Logging {
         String privKeyFile = getPrivKeyFile(); 
         _tunnel.runIrcServer(new String[] { targetHost, targetPort, privKeyFile }, this);
     }
-    
+
     private void setListenOn() {
         String listenOn = getListenOnInterface();
         if ( (listenOn != null) && (listenOn.length() > 0) ) {
             _tunnel.runListenOn(new String[] { listenOn }, this);
         }
     }
-    
+
     /**
      *  These are the ones stored with a prefix of "option."
      *  Defaults in config properties are not honored.
@@ -719,7 +720,7 @@ public class TunnelController implements Logging {
             opts.setProperty(PROP_TARGET_PORT, targetport);
         _tunnel.setClientOptions(opts);
     }
-    
+
     private void setI2CPOptions() {
         String host = getI2CPHost();
         if ( (host != null) && (host.length() > 0) ) 
@@ -778,12 +779,12 @@ public class TunnelController implements Logging {
         release(sessions);
         changeState(TunnelState.DESTROYED);
     }
-    
+
     public void restartTunnel() {
         stopTunnel();
         startTunnel();
     }
-    
+
     /**
      *  As of 0.9.1, updates the options on an existing session
      */
@@ -834,6 +835,8 @@ public class TunnelController implements Logging {
                     _config.setProperty(OPT_SIG_TYPE, PREFERRED_SIGTYPE.name());
             }
             if (!isClient(type)) {
+                _tunnel.filterDefinition = _config.getProperty(PROP_FILTER);
+              
                 String p1 = _config.getProperty(OPT_MAX_CONNS_MIN, "0");
                 String p2 = _config.getProperty(OPT_MAX_CONNS_HOUR, "0");
                 String p3 = _config.getProperty(OPT_MAX_CONNS_DAY, "0");
@@ -931,6 +934,11 @@ public class TunnelController implements Logging {
     public String getDescription() { return _config.getProperty(PROP_DESCR); }
     public String getI2CPHost() { return _config.getProperty(PROP_I2CP_HOST); }
     public String getI2CPPort() { return _config.getProperty(PROP_I2CP_PORT); }
+    /**
+     *  Absolute path to filter definition file
+     *  @since 0.9.40
+     */
+    public String getFilter() { return _config.getProperty(PROP_FILTER); }
 
     /**
      *  Is it a client or server in the UI and I2P side?
@@ -1046,7 +1054,7 @@ public class TunnelController implements Logging {
             return dest.toBase64();
         return null;
     }
-    
+
     /**
      *  Returns null if not running.
      *  @return "{52 chars}.b32.i2p" or null
@@ -1057,7 +1065,7 @@ public class TunnelController implements Logging {
             return dest.toBase32();
         return null;
     }
-    
+
     /**
      *  Returns null if not running.
      *  @return Destination or null
@@ -1075,7 +1083,22 @@ public class TunnelController implements Logging {
         }
         return null;
     }
-    
+
+    /**
+     *  Returns false if not running.
+     *  @return true if offline keys or not running
+     *  @since 0.9.40
+     */
+    public boolean getIsOfflineKeys() {
+        if (_tunnel != null) {
+            List<I2PSession> sessions = _tunnel.getSessions();
+            if (!sessions.isEmpty())
+                return sessions.get(0).isOffline();
+        }
+        return false;
+    }
+
+    // TODO synch
     public boolean getIsRunning() { return _state == TunnelState.RUNNING; }
     public boolean getIsStarting() { return _state == TunnelState.START_ON_LOAD || _state == TunnelState.STARTING; }
 
@@ -1097,7 +1120,7 @@ public class TunnelController implements Logging {
     private synchronized void changeState(TunnelState state) {
         _state = state;
     }
-    
+
     /**
      *  A text description of the tunnel.
      *  @deprecated unused
@@ -1119,7 +1142,7 @@ public class TunnelController implements Logging {
             buf.append("Unknown type ").append(type);
        ****/
     }
-    
+
   /****
     private void getHttpClientSummary(StringBuilder buf) {
         String description = getDescription();
@@ -1141,7 +1164,7 @@ public class TunnelController implements Logging {
             buf.append("Outproxy: ").append(proxies).append("<br />\n");
         getOptionSummary(buf);
     }
-    
+
     private void getClientSummary(StringBuilder buf) {
         String description = getDescription();
         if ( (description != null) && (description.trim().length() > 0) )
@@ -1158,7 +1181,7 @@ public class TunnelController implements Logging {
         buf.append("<br />\n");
         getOptionSummary(buf);
     }
-    
+
     private void getServerSummary(StringBuilder buf) {
         String description = getDescription();
         if ( (description != null) && (description.trim().length() > 0) )
@@ -1169,7 +1192,7 @@ public class TunnelController implements Logging {
         buf.append("Private destination loaded from ").append(getPrivKeyFile()).append("<br />\n");
         getOptionSummary(buf);
     }
-    
+
     private void getHttpServerSummary(StringBuilder buf) {
         String description = getDescription();
         if ( (description != null) && (description.trim().length() > 0) )
@@ -1181,7 +1204,7 @@ public class TunnelController implements Logging {
         buf.append("Private destination loaded from ").append(getPrivKeyFile()).append("<br />\n");
         getOptionSummary(buf);
     }
-    
+
     private void getOptionSummary(StringBuilder buf) {
         String opts = getClientOptions();
         if ( (opts != null) && (opts.length() > 0) )
@@ -1213,7 +1236,7 @@ public class TunnelController implements Logging {
         }
     }
   ****/
-    
+
     /**
      *
      */
@@ -1226,7 +1249,7 @@ public class TunnelController implements Logging {
         if (_log.shouldLog(Log.INFO))
             _log.info(s);
     }
-    
+
     /**
      * Pull off any messages that the I2PTunnel has produced 
      *
