@@ -14,6 +14,7 @@ import net.i2p.I2PAppContext;
 import net.i2p.I2PException;
 import net.i2p.client.I2PClient;
 import net.i2p.crypto.SigType;
+import net.i2p.data.Base64;
 import net.i2p.data.DataHelper;
 import net.i2p.data.Destination;
 import net.i2p.data.Hash;
@@ -447,6 +448,26 @@ public class GeneralHelper {
         return null;
     }
 
+    /**
+     *  Works even if tunnel is not running.
+     *  @return true if offline keys
+     *  @since 0.9.40
+     */
+    public boolean isOfflineKeys(int tunnel) {
+        TunnelController tun = getController(tunnel);
+        if (tun != null) {
+            if (tun.getIsRunning())
+                return tun.getIsOfflineKeys();
+            // do this the hard way
+            File keyFile = tun.getPrivateKeyFile();
+            if (keyFile != null) {
+                PrivateKeyFile pkf = new PrivateKeyFile(keyFile);
+                return pkf.isOffline();
+            }
+        }
+        return false;
+    }
+
     public boolean shouldStartAutomatically(int tunnel) {
         TunnelController tun = getController(tunnel);
         return tun != null ? tun.getStartOnLoad() : false;
@@ -537,6 +558,71 @@ public class GeneralHelper {
     public boolean getEncrypt(int tunnel) {
         return getBooleanProperty(tunnel, "i2cp.encryptLeaseSet");
     }
+    
+    /**
+     *  @since 0.9.40
+     */
+    public int getEncryptMode(int tunnel) {
+        if (getEncrypt(tunnel))
+            return 1;
+        String lstype = getProperty(tunnel, "i2cp.leaseSetType", "1");
+        if (lstype.equals("5")) {
+            int rv;
+            String authType = getProperty(tunnel, "i2cp.leaseSetAuthType", "0");
+            if (authType.equals("2")) {
+                if (getProperty(tunnel, "i2cp.leaseSetClient.psk.0", null) != null) {
+                    // per-client PSK key
+                    rv = 6;
+                } else {
+                    // shared PSK key
+                    rv = 4;
+                }
+            } else if (authType.equals("1")) {
+                rv = 8;
+            } else {
+                rv = 2;
+            }
+
+            String pw = getBlindedPassword(tunnel);
+            if (pw != null && pw.length() > 0)
+                rv++;
+            return rv;
+        } else if (lstype.equals("3")) {
+            return 10;
+        }
+        return 0;
+    }
+    
+    /**
+     *  @since 0.9.40
+     */
+    public String getBlindedPassword(int tunnel) {
+        String rv = getProperty(tunnel, "i2cp.leaseSetSecret", null);
+        if (rv != null)
+            rv = DataHelper.getUTF8(Base64.decode(rv));
+        if (rv == null)
+            rv = "";
+        return rv;
+    }
+    
+    /**
+     *  List of b64 name : b64key
+     *  Pubkeys for DH, privkeys for PSK
+     *  @param isDH true for DH, false for PSK
+     *  @return non-null
+     *  @since 0.9.41
+     */
+    public List<String> getClientAuths(int tunnel, boolean isDH) {
+        List<String> rv = new ArrayList<String>(4);
+        String pfx = isDH ? "i2cp.leaseSetClient.dh." : "i2cp.leaseSetClient.psk.";
+        int i = 0;
+        String p;
+        while ((p = getProperty(tunnel, pfx + i, null)) != null) {
+             rv.add(p);
+             i++;
+        }
+        return rv;
+    }
 
     /**
      *  @param newTunnelType used if tunnel &lt; 0
@@ -621,6 +707,19 @@ public class GeneralHelper {
     public String getAccessList(int tunnel) {
         return getProperty(tunnel, "i2cp.accessList", "").replace(",", "\n");
     }
+
+    /**
+     *  @since 0.9.40
+     */
+    public String getFilterDefinition(int tunnel) {
+        TunnelController tunnelController = getController(tunnel);
+        if (tunnelController != null) {
+            String filter = tunnelController.getFilter();
+            if (filter != null)
+                return filter;
+        }
+        return "";
+    }
     
     public String getJumpList(int tunnel) {
         return getProperty(tunnel, I2PTunnelHTTPClient.PROP_JUMP_SERVERS,
@@ -796,6 +895,8 @@ public class GeneralHelper {
                 if ((!isMD5Proxy) &&
                     TunnelConfig._nonProxyNoShowSet.contains(key))
                     continue;
+                if (key.startsWith("i2cp.leaseSetClient."))
+                    continue;
                 sorted.put(key, (String)e.getValue());
             }
             if (sorted.isEmpty())
