@@ -212,6 +212,31 @@ public class EditBean extends IndexBean {
         return _helper.getEncrypt(tunnel);
     }
     
+    /**
+     *  @since 0.9.40
+     */
+    public String getEncryptMode(int tunnel) {
+        return Integer.toString(_helper.getEncryptMode(tunnel));
+    }
+    
+    /**
+     *  @since 0.9.40
+     */
+    public String getBlindedPassword(int tunnel) {
+        return _helper.getBlindedPassword(tunnel);
+    }
+    
+    /**
+     *  List of b64 name : b64key
+     *  Pubkeys for DH, privkeys for PSK
+     *  @param isDH true for DH, false for PSK
+     *  @return non-null
+     *  @since 0.9.41
+     */
+    public List<String> getClientAuths(int tunnel, boolean isDH) {
+        return _helper.getClientAuths(tunnel, isDH);
+    }
+    
     /**
      *  @param newTunnelType used if tunnel &lt; 0
      *  @since 0.9.12
@@ -270,6 +295,13 @@ public class EditBean extends IndexBean {
     public String getAccessList(int tunnel) {
         return _helper.getAccessList(tunnel);
     }
+
+    /**
+     *  @since 0.9.40
+     */
+    public String getFilterDefinition(int tunnel) {
+        return _helper.getFilterDefinition(tunnel);
+    }
     
     public String getJumpList(int tunnel) {
         return _helper.getJumpList(tunnel);
