@@ -36,6 +36,7 @@ import net.i2p.util.SecureDirectory;
 import net.i2p.util.SimpleScheduler;
 import net.i2p.util.SimpleTimer;
 import net.i2p.util.SimpleTimer2;
+import net.i2p.util.SystemVersion;
 import net.i2p.util.I2PProperties.I2PPropertyCallback;
 
 /**
@@ -694,10 +695,7 @@ public class I2PAppContext {
     private void initializeAESEngine() {
         synchronized (_lock7) {
             if (_AESEngine == null) {
-                if ("off".equals(getProperty("i2p.encryption", "on")))
-                    _AESEngine = new AESEngine(this);
-                else
-                    _AESEngine = new CryptixAESEngine(this);
+                _AESEngine = new CryptixAESEngine(this);
             }
             _AESEngineInitialized = true;
         }
@@ -723,6 +721,20 @@ public class I2PAppContext {
         }
     }
 
+    /**
+     * Overwrites the LogManager instance to be used by the router.
+     * This should only be called after the Router is instantiated but
+     * before it is started.  Calling this at any other time can have
+     * unpredictable side effects.
+     * @since 0.9.41
+     */
+    public void setLogManager(LogManager logManager) {
+        synchronized (_lock8) {
+            _logManager = logManager;
+            _logManagerInitialized = true;
+        }
+    }
+
     /** 
      * There is absolutely no good reason to make this context specific, 
      * other than for consistency, and perhaps later we'll want to 
@@ -931,7 +943,7 @@ public class I2PAppContext {
      *  @since 0.8.8
      */
     public boolean hasWrapper() {
-        return System.getProperty("wrapper.version") != null;
+        return SystemVersion.hasWrapper();
     }
 
     /**
