@@ -9,6 +9,7 @@ package net.i2p.client.impl;
  *
  */
 
+import java.util.List;
 import java.util.Properties;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
@@ -65,7 +66,9 @@ class I2CPMessageProducer {
         "i2cp.closeIdleTime", "i2cp.closeOnIdle", "i2cp.encryptLeaseSet",
         "i2cp.gzip", "i2cp.leaseSetKey", "i2cp.leaseSetPrivateKey",
         "i2cp.leaseSetSigningPrivateKey", "i2cp.reduceIdleTime", "i2cp.reduceOnIdle",
-        I2PSessionImpl.PROP_ENABLE_SSL, I2PClient.PROP_TCP_HOST, I2PClient.PROP_TCP_PORT
+        I2PSessionImpl.PROP_ENABLE_SSL, I2PClient.PROP_TCP_HOST, I2PClient.PROP_TCP_PORT,
+        // long and shouldn't be passed through
+        "i2p.reseedURL"
     };
 
 
@@ -119,8 +122,14 @@ class I2CPMessageProducer {
         updateBandwidth(session);
         CreateSessionMessage msg = new CreateSessionMessage();
         SessionConfig cfg = new SessionConfig(session.getMyDestination());
-        cfg.setOptions(getRouterOptions(session));
-        if (session.isOffline()) {
+        Properties p = getRouterOptions(session);
+        boolean isOffline = session.isOffline();
+        if (isOffline) {
+            if (!p.containsKey(RequestLeaseSetMessageHandler.PROP_LS_TYPE))
+                p.setProperty(RequestLeaseSetMessageHandler.PROP_LS_TYPE, "3");
+        }
+        cfg.setOptions(p);
+        if (isOffline) {
             cfg.setOfflineSignature(session.getOfflineExpiration(),
                                     session.getTransientSigningPublicKey(),
                                     session.getOfflineSignature());
@@ -361,18 +370,24 @@ class I2CPMessageProducer {
      * This method is misnamed, it does not create the LeaseSet,
      * the caller does that.
      *
+     * @param signingPriv ignored for LS2
      */
     public void createLeaseSet(I2PSessionImpl session, LeaseSet leaseSet, SigningPrivateKey signingPriv,
-                               PrivateKey priv) throws I2PSessionException {
+                               List<PrivateKey> privs) throws I2PSessionException {
         CreateLeaseSetMessage msg;
         int type = leaseSet.getType();
-        if (type == DatabaseEntry.KEY_TYPE_LEASESET)
+        if (type == DatabaseEntry.KEY_TYPE_LEASESET) {
             msg = new CreateLeaseSetMessage();
-        else
-            msg = new CreateLeaseSet2Message();
+            msg.setPrivateKey(privs.get(0));
+            msg.setSigningPrivateKey(signingPriv);
+        } else {
+            CreateLeaseSet2Message msg2 = new CreateLeaseSet2Message();
+            for (PrivateKey priv : privs) {
+                msg2.addPrivateKey(priv);
+            }
+            msg = msg2;
+        }
         msg.setLeaseSet(leaseSet);
-        msg.setPrivateKey(priv);
-        msg.setSigningPrivateKey(signingPriv);
         SessionId sid = session.getSessionId();
         if (sid == null) {
             _log.error(session.toString() + " create LS w/o session", new Exception());
