@@ -35,6 +35,7 @@ import com.southernstorm.noise.crypto.x25519.Curve25519;
 import net.i2p.I2PAppContext;
 import net.i2p.crypto.eddsa.EdDSAPrivateKey;
 import net.i2p.crypto.eddsa.EdDSAPublicKey;
+import net.i2p.crypto.eddsa.RedKeyPairGenerator;
 import net.i2p.crypto.eddsa.spec.EdDSAPublicKeySpec;
 import net.i2p.crypto.provider.I2PProvider;
 import net.i2p.data.Hash;
@@ -289,7 +290,11 @@ public final class KeyGenerator {
             return generateSigningKeys();
         java.security.KeyPair kp;
         if (type.getBaseAlgorithm() == SigAlgo.EdDSA) {
-            net.i2p.crypto.eddsa.KeyPairGenerator kpg = new net.i2p.crypto.eddsa.KeyPairGenerator();
+            net.i2p.crypto.eddsa.KeyPairGenerator kpg;
+            if (type == SigType.RedDSA_SHA512_Ed25519)
+                kpg = new RedKeyPairGenerator();
+            else
+                kpg = new net.i2p.crypto.eddsa.KeyPairGenerator();
             kpg.initialize(type.getParams(), _context.random());
             kp = kpg.generateKeyPair();
         } else {
@@ -466,13 +471,13 @@ public final class KeyGenerator {
         else
             System.out.println(type + " private-to-public test FAILED");
         //System.out.println("privkey " + keys[1]);
-          MessageDigest md = type.getDigestInstance();
+        MessageDigest md = type.getDigestInstance();
         for (int i = 0; i < runs; i++) {
             RandomSource.getInstance().nextBytes(src);
-              md.update(src);
-              byte[] sha = md.digest();
-              SimpleDataStructure hash = type.getHashInstance();
-              hash.setData(sha);
+            md.update(src);
+            byte[] sha = md.digest();
+            SimpleDataStructure hash = type.getHashInstance();
+            hash.setData(sha);
             long start = System.nanoTime();
             Signature sig = DSAEngine.getInstance().sign(src, privkey);
             Signature sig2 = DSAEngine.getInstance().sign(hash, privkey);
@@ -487,16 +492,18 @@ public final class KeyGenerator {
             stime += mid - start;
             vtime += end - mid;
             if (!ok)
-                throw new GeneralSecurityException(type + " V(S(data)) fail");
+                throw new GeneralSecurityException(type + " V(S(data)) fail on run " + i);
             if (!ok2)
-                throw new GeneralSecurityException(type + " V(S(H(data))) fail");
+                throw new GeneralSecurityException(type + " V(S(H(data))) fail on run" + i);
         }
         stime /= 1000*1000;
         vtime /= 1000*1000;
+        // we do two of each per run above
+        int runs2 = runs * 2;
         System.out.println(type + " sign/verify " + runs + " times: " + (vtime+stime) + " ms = " +
-                           (((double) stime) / runs) + " each sign, " +
-                           (((double) vtime) / runs) + " each verify, " +
-                           (((double) (stime + vtime)) / runs) + " s+v");
+                           (((double) stime) / runs2) + " each sign, " +
+                           (((double) vtime) / runs2) + " each verify, " +
+                           (((double) (stime + vtime)) / runs2) + " s+v");
     }
 
 /******
