@@ -12,10 +12,12 @@ import net.i2p.crypto.eddsa.math.GroupElement;
  */
 public class EdDSAPublicKeySpec implements KeySpec {
     private final GroupElement A;
-    private final GroupElement Aneg;
+    private GroupElement Aneg;
     private final EdDSAParameterSpec spec;
 
     /**
+     *  @param pk the public key
+     *  @param spec the parameter specification for this key
      *  @throws IllegalArgumentException if key length is wrong
      */
     public EdDSAPublicKeySpec(byte[] pk, EdDSAParameterSpec spec) {
@@ -23,16 +25,11 @@ public class EdDSAPublicKeySpec implements KeySpec {
             throw new IllegalArgumentException("public-key length is wrong");
 
         this.A = new GroupElement(spec.getCurve(), pk);
-        // Precompute -A for use in verification.
-        this.Aneg = A.negate();
-        Aneg.precompute(false);
         this.spec = spec;
     }
 
     public EdDSAPublicKeySpec(GroupElement A, EdDSAParameterSpec spec) {
         this.A = A;
-        this.Aneg = A.negate();
-        Aneg.precompute(false);
         this.spec = spec;
     }
 
@@ -41,7 +38,13 @@ public class EdDSAPublicKeySpec implements KeySpec {
     }
 
     public GroupElement getNegativeA() {
-        return Aneg;
+        // Only read Aneg once, otherwise read re-ordering might occur between here and return. Requires all GroupElement's fields to be final.
+        GroupElement ourAneg = Aneg;
+        if(ourAneg == null) {
+            ourAneg = A.negate();
+            Aneg = ourAneg;
+        }
+        return ourAneg;
     }
 
     public EdDSAParameterSpec getParams() {
