@@ -20,12 +20,14 @@ import com.nettgryppa.security.HashCash;
 
 import gnu.getopt.Getopt;
 
+import net.i2p.I2PAppContext;
 import net.i2p.I2PException;
 import net.i2p.client.I2PClient;
 import net.i2p.client.I2PClientFactory;
 import net.i2p.client.I2PSession;
 import net.i2p.client.I2PSessionException;
 import net.i2p.client.naming.HostTxtEntry;
+import net.i2p.crypto.Blinding;
 import net.i2p.crypto.DSAEngine;
 import net.i2p.crypto.KeyGenerator;
 import net.i2p.crypto.SigType;
@@ -620,24 +622,17 @@ public class PrivateKeyFile {
     
     //// offline methods
 
-    /**
-     *  Constant time
-     *  @since 0.9.38
-     */
-    private static boolean isOffline(SigningPrivateKey spk) {
-        byte b = 0;
-        byte[] data = spk.getData();
-        for (int i = 0; i < data.length; i++) {
-            b |= data[i];
-        }
-        return b == 0;
-    }
-
     /**
      *  Does this session have offline and transient keys?
      *  @since 0.9.38
      */
     public boolean isOffline() {
+        try {
+            // call this to force initialization
+            getDestination();
+        } catch (Exception e) {
+            return false;
+        }
         return _offlineSignature != null;
     }
 
@@ -646,7 +641,7 @@ public class PrivateKeyFile {
      *  @since 0.9.38
      */
     public void setOfflineData(long expires, SigningPublicKey transientPub, Signature sig, SigningPrivateKey transientPriv) {
-        if (!isOffline(signingPrivKey)) {
+        if (!signingPrivKey.isOffline()) {
             SigType type = getSigningPrivKey().getType();
             byte[] dbytes = new byte[type.getPrivkeyLen()];
             signingPrivKey = new SigningPrivateKey(type, dbytes);
@@ -768,6 +763,15 @@ public class PrivateKeyFile {
         s.append(this.dest != null ? this.dest.toBase64() : "null");
         s.append("\nB32: ");
         s.append(this.dest != null ? this.dest.toBase32() : "null");
+        if (dest != null) {
+            SigningPublicKey spk = dest.getSigningPublicKey();
+            SigType type = spk.getType();
+            if (type == SigType.EdDSA_SHA512_Ed25519 ||
+                type == SigType.RedDSA_SHA512_Ed25519) {
+                I2PAppContext ctx = I2PAppContext.getGlobalContext();
+                s.append("\nBlinded B32: ").append(Blinding.encode(spk));
+            }
+        }
         s.append("\nContains: ");
         s.append(this.dest);
         s.append("\nPrivate Key: ");
