@@ -12,13 +12,18 @@ import java.util.List;
 import java.util.Properties;
 
 import net.i2p.CoreVersion;
+import net.i2p.crypto.EncType;
 import net.i2p.crypto.SigType;
+import net.i2p.data.Base64;
 import net.i2p.data.DatabaseEntry;
 import net.i2p.data.DataHelper;
 import net.i2p.data.Destination;
+import net.i2p.data.EncryptedLeaseSet;
 import net.i2p.data.Hash;
 import net.i2p.data.LeaseSet;
+import net.i2p.data.LeaseSet2;
 import net.i2p.data.Payload;
+import net.i2p.data.PrivateKey;
 import net.i2p.data.PublicKey;
 import net.i2p.data.i2cp.BandwidthLimitsMessage;
 import net.i2p.data.i2cp.CreateLeaseSetMessage;
@@ -268,13 +273,44 @@ class ClientMessageEventListener implements I2CPMessageReader.I2CPMessageEventLi
             }
         }
         props.putAll(inProps);
-        if ("7".equals(props.getProperty("i2cp.leaseSetType"))) {
+        String senc = props.getProperty("i2cp.leaseSetEncType");
+        if (senc != null) {
+            String[] senca = DataHelper.split(senc, ",");
+            for (String sencaa : senca) {
+                EncType type = EncType.parseEncType(sencaa);
+                if (type != null) {
+                    if (!type.isAvailable()) {
+                        String msg = "Unsupported crypto type: " + type;
+                        _log.error(msg);
+                        _runner.disconnectClient(msg);
+                        return;
+                    }
+                } else {
+                    String msg = "Unsupported crypto type: " + sencaa;
+                    _log.error(msg);
+                    _runner.disconnectClient(msg);
+                    return;
+                }
+            }
+        }
+        String lsType = props.getProperty("i2cp.leaseSetType");
+        if ("5".equals(lsType)) {
+            SigType stype = dest.getSigningPublicKey().getType();
+            if (stype != SigType.EdDSA_SHA512_Ed25519 &&
+                stype != SigType.RedDSA_SHA512_Ed25519) {
+                _runner.disconnectClient("Invalid sig type for encrypted LS");
+                return;
+            }
+        } else if ("7".equals(lsType)) {
             // Prevent tunnel builds for Meta LS
             // more TODO
             props.setProperty("inbound.length", "0");
             props.setProperty("outbound.length", "0");
             props.setProperty("inbound.lengthVariance", "0");
             props.setProperty("outbound.lengthVariance", "0");
+        } else if (lsType == null && props.getProperty(SessionConfig.PROP_OFFLINE_SIGNATURE) != null) {
+            // force type 3
+            props.setProperty("i2cp.leaseSetType", "3");
         }
         cfg.setOptions(props);
         // this sets the session id
@@ -488,7 +524,15 @@ class ClientMessageEventListener implements I2CPMessageReader.I2CPMessageEventLi
         }
         int type = ls.getType();
         if (type != DatabaseEntry.KEY_TYPE_META_LS2 &&
-            (message.getPrivateKey() == null || message.getSigningPrivateKey() == null)) {
+            type != DatabaseEntry.KEY_TYPE_ENCRYPTED_LS2 &&
+            message.getPrivateKey() == null) {
+            if (_log.shouldLog(Log.ERROR))
+                _log.error("Null private keys: " + message);
+            _runner.disconnectClient("Invalid CreateLeaseSetMessage - null private keys");
+            return;
+        }
+        if (type == DatabaseEntry.KEY_TYPE_LEASESET && message.getSigningPrivateKey() == null) {
+            // revocation keys only in LS1
             if (_log.shouldLog(Log.ERROR))
                 _log.error("Null private keys: " + message);
             _runner.disconnectClient("Invalid CreateLeaseSetMessage - null private keys");
@@ -505,12 +549,47 @@ class ClientMessageEventListener implements I2CPMessageReader.I2CPMessageEventLi
             return;
         }
         Destination dest = cfg.getDestination();
-        Destination ndest = ls.getDestination();
-        if (!dest.equals(ndest)) {
-            if (_log.shouldLog(Log.ERROR))
-                _log.error("Different destination in LS");
-            _runner.disconnectClient("Different destination in LS");
-            return;
+        if (type == DatabaseEntry.KEY_TYPE_ENCRYPTED_LS2) {
+            // so we can decrypt it
+            // secret must be set before destination
+            String secret = cfg.getOptions().getProperty("i2cp.leaseSetSecret");
+            if (secret != null) {
+                EncryptedLeaseSet encls = (EncryptedLeaseSet) ls;
+                secret = DataHelper.getUTF8(Base64.decode(secret));
+                encls.setSecret(secret);
+            }
+            try {
+                ls.setDestination(dest);
+            } catch (RuntimeException re) {
+                if (_log.shouldError())
+                    _log.error("Error decrypting leaseset from client", re);
+                _runner.disconnectClient(re.toString());
+                return;
+            }
+            // per-client auth
+            // we have to do this before verifySignature()
+            String pk = cfg.getOptions().getProperty("i2cp.leaseSetPrivKey");
+            if (pk != null) {
+                byte[] priv = Base64.decode(pk);
+                PrivateKey privkey = new PrivateKey(EncType.ECIES_X25519, priv);
+                EncryptedLeaseSet encls = (EncryptedLeaseSet) ls;
+                encls.setClientPrivateKey(privkey);
+            }
+            // we have to do this before checking encryption keys below
+            if (!ls.verifySignature()) {
+                if (_log.shouldError())
+                    _log.error("Error decrypting leaseset from client");
+                _runner.disconnectClient("Error decrypting leaseset from client");
+                return;
+            }
+        } else {
+            Destination ndest = ls.getDestination();
+            if (!dest.equals(ndest)) {
+                if (_log.shouldLog(Log.ERROR))
+                    _log.error("Different destination in LS");
+                _runner.disconnectClient("Different destination in LS");
+                return;
+            }
         }
         if (type != DatabaseEntry.KEY_TYPE_META_LS2) {
             LeaseSetKeys keys = _context.keyManager().getKeys(dest);
@@ -519,30 +598,78 @@ class ClientMessageEventListener implements I2CPMessageReader.I2CPMessageEventLi
                 // Verify and register crypto keys if new or if changed
                 // Private crypto key should never change, and if it does,
                 // one of the checks below will fail
-                PublicKey pk;
-                try {
-                    pk = message.getPrivateKey().toPublic();
-                } catch (IllegalArgumentException iae) {
-                    if (_log.shouldLog(Log.ERROR))
-                        _log.error("Bad private key in LS");
-                    _runner.disconnectClient("Bad private key in LS");
-                    return;
-                }
-                if (!pk.equals(ls.getEncryptionKey())) {
-                    if (_log.shouldLog(Log.ERROR))
-                        _log.error("Private/public crypto key mismatch in LS");
-                    _runner.disconnectClient("Private/public crypto key mismatch in LS");
-                    return;
+                if (type == DatabaseEntry.KEY_TYPE_LEASESET) {
+                    // LS1
+                    PublicKey pk;
+                    try {
+                        pk = message.getPrivateKey().toPublic();
+                    } catch (IllegalArgumentException iae) {
+                        if (_log.shouldLog(Log.ERROR))
+                            _log.error("Bad private key in LS");
+                        _runner.disconnectClient("Bad private key in LS");
+                        return;
+                    }
+                    if (!pk.equals(ls.getEncryptionKey())) {
+                        if (_log.shouldLog(Log.ERROR))
+                            _log.error("Private/public crypto key mismatch in LS");
+                        _runner.disconnectClient("Private/public crypto key mismatch in LS");
+                        return;
+                    }
+                } else {
+                    // LS2
+                    LeaseSet2 ls2 = (LeaseSet2) ls;
+                    CreateLeaseSet2Message msg2 = (CreateLeaseSet2Message) message;
+                    List<PublicKey> eks = ls2.getEncryptionKeys();
+                    List<PrivateKey> pks = msg2.getPrivateKeys();
+                    for (int i = 0; i < eks.size(); i++) {
+                        PublicKey ek = eks.get(i);
+                        PublicKey pk;
+                        try {
+                            pk = pks.get(i).toPublic();
+                        } catch (IllegalArgumentException iae) {
+                            if (_log.shouldLog(Log.ERROR))
+                                _log.error("Bad private key in LS: " + i);
+                            _runner.disconnectClient("Bad private key in LS: " + i);
+                            return;
+                        }
+                        if (!pk.equals(ek)) {
+                            if (_log.shouldLog(Log.ERROR))
+                                _log.error("Private/public crypto key mismatch in LS for key: " + i);
+                            _runner.disconnectClient("Private/public crypto key mismatch in LS for key: " + i);
+                            return;
+                        }
+                    }
                 }
                 // just register new SPK, don't verify, unused
                 _context.keyManager().registerKeys(dest, message.getSigningPrivateKey(), message.getPrivateKey());
-            } else if (!message.getSigningPrivateKey().equals(keys.getRevocationKey())) {
+            } else if (message.getSigningPrivateKey() != null &&
+                       !message.getSigningPrivateKey().equals(keys.getRevocationKey())) {
                 // just register new SPK, don't verify, unused
                 _context.keyManager().registerKeys(dest, message.getSigningPrivateKey(), message.getPrivateKey());
             }
         }
         try {
+            if (type == DatabaseEntry.KEY_TYPE_ENCRYPTED_LS2) {
+                // so the client manager knows it's a local hash
+                // we could put this in runner.leaseSetCreated(), but
+                // need to set it before calling publish()
+                Hash newHash = ls.getHash();
+                boolean ok = _runner.registerEncryptedLS(newHash);
+                if (!ok) {
+                    _runner.disconnectClient("Duplicate hash of encrypted LS2");
+                    return;
+                }
+            }
+            if (_log.shouldDebug())
+                _log.debug("Publishing: " + ls);
             _context.netDb().publish(ls);
+            if (type == DatabaseEntry.KEY_TYPE_ENCRYPTED_LS2) {
+                // store the decrypted ls also
+                EncryptedLeaseSet encls = (EncryptedLeaseSet) ls;
+                if (_log.shouldDebug())
+                    _log.debug("Storing decrypted: " + encls.getDecryptedLeaseSet());
+                _context.netDb().store(dest.getHash(), encls.getDecryptedLeaseSet());
+            }
         } catch (IllegalArgumentException iae) {
             if (_log.shouldLog(Log.ERROR))
                 _log.error("Invalid leaseset from client", iae);
