@@ -53,7 +53,7 @@ abstract class StoreJob extends JobImpl {
     private final static int PARALLELIZATION = 4; // how many sent at a time
     private final static int REDUNDANCY = 4; // we want the data sent to 6 peers
     private final static int STORE_PRIORITY = OutNetMessage.PRIORITY_MY_NETDB_STORE;
-    
+
     /**
      * Send a data structure to the floodfills
      * 
@@ -89,9 +89,12 @@ abstract class StoreJob extends JobImpl {
             else
                 _connectMask = ConnectChecker.ANY_V4;
         }
+        if (_log.shouldLog(Log.DEBUG))
+            _log.debug(getJobId() + ": New store job for\n" + data, new Exception("I did it"));
     }
 
     public String getName() { return "Kademlia NetDb Store";}
+
     public void runJob() {
         sendNext();
     }
@@ -181,7 +184,11 @@ abstract class StoreJob extends JobImpl {
             int queued = 0;
             int skipped = 0;
             int type = _state.getData().getType();
-            boolean isls2 = DatabaseEntry.isLeaseSet(type) && type != DatabaseEntry.KEY_TYPE_LEASESET;
+            final boolean isls = DatabaseEntry.isLeaseSet(type);
+            final boolean isls2 = isls && type != DatabaseEntry.KEY_TYPE_LEASESET;
+            final SigType lsSigType = (isls && type != DatabaseEntry.KEY_TYPE_ENCRYPTED_LS2) ?
+                                      _state.getData().getKeysAndCert().getSigningPublicKey().getType() :
+                                      null;
             for (Hash peer : closestHashes) {
                 DatabaseEntry ds = _facade.getDataStore().get(peer);
                 if ( (ds == null) || !(ds.getType() == DatabaseEntry.KEY_TYPE_ROUTERINFO) ) {
@@ -194,6 +201,13 @@ abstract class StoreJob extends JobImpl {
                         _log.info(getJobId() + ": Skipping old router " + peer);
                     _state.addSkipped(peer);
                     skipped++;
+                } else if ((type == DatabaseEntry.KEY_TYPE_ENCRYPTED_LS2 ||
+                            lsSigType == SigType.RedDSA_SHA512_Ed25519) &&
+                           !shouldStoreEncLS2To((RouterInfo)ds)) {
+                    if (_log.shouldInfo())
+                        _log.info(getJobId() + ": Skipping router that doesn't support EncLS2/RedDSA " + peer);
+                    _state.addSkipped(peer);
+                    skipped++;
                 } else if (isls2 &&
                            !shouldStoreLS2To((RouterInfo)ds)) {
                     if (_log.shouldLog(Log.INFO))
@@ -332,7 +346,9 @@ abstract class StoreJob extends JobImpl {
             getContext().statManager().addRateData("netDb.storeLeaseSetSent", 1);
             // if it is an encrypted leaseset...
             if (getContext().keyRing().get(msg.getKey()) != null)
-                sendStoreThroughGarlic(msg, peer, expiration);
+                sendStoreThroughExploratory(msg, peer, expiration);
+            else if (msg.getEntry().getType() == DatabaseEntry.KEY_TYPE_META_LS2)
+                sendWrappedStoreThroughExploratory(msg, peer, expiration);
             else
                 sendStoreThroughClient(msg, peer, expiration);
         } else {
@@ -341,7 +357,7 @@ abstract class StoreJob extends JobImpl {
             if (_connectChecker.canConnect(_connectMask, peer))
                 sendDirect(msg, peer, expiration);
             else
-                sendStoreThroughGarlic(msg, peer, expiration);
+                sendStoreThroughExploratory(msg, peer, expiration);
         }
     }
 
@@ -373,12 +389,13 @@ abstract class StoreJob extends JobImpl {
     }
     
     /**
-     * This is misnamed, it means sending it out through an exploratory tunnel,
+     * Send it out through an exploratory tunnel,
      * with the reply to come back through an exploratory tunnel.
      * There is no garlic encryption added.
      *
+     * @since 0.9.41 renamed from sendStoreThroughGarlic()
      */
-    private void sendStoreThroughGarlic(DatabaseStoreMessage msg, RouterInfo peer, long expiration) {
+    private void sendStoreThroughExploratory(DatabaseStoreMessage msg, RouterInfo peer, long expiration) {
         long token = 1 + getContext().random().nextLong(I2NPMessage.MAX_ID_VALUE);
         
         Hash to = peer.getIdentity().getHash();
@@ -410,7 +427,7 @@ abstract class StoreJob extends JobImpl {
             StoreMessageSelector selector = new StoreMessageSelector(getContext(), getJobId(), peer, token, expiration);
     
             if (_log.shouldLog(Log.DEBUG))
-                _log.debug("sending store to " + peer.getIdentity().getHash() + " through " + outTunnel + ": " + msg);
+                _log.debug(getJobId() + ": sending store to " + peer.getIdentity().getHash() + " through " + outTunnel + ": " + msg);
             getContext().messageRegistry().registerPending(selector, onReply, onFail);
             getContext().tunnelDispatcher().dispatchOutbound(msg, outTunnel.getSendTunnelId(0), null, to);
         } else {
@@ -438,7 +455,13 @@ abstract class StoreJob extends JobImpl {
      */
     private void sendStoreThroughClient(DatabaseStoreMessage msg, RouterInfo peer, long expiration) {
         long token = 1 + getContext().random().nextLong(I2NPMessage.MAX_ID_VALUE);
-        Hash client = msg.getKey();
+        Hash client;
+        if (msg.getEntry().getType() == DatabaseEntry.KEY_TYPE_ENCRYPTED_LS2) {
+            // get the real client hash
+            client = ((LeaseSet)msg.getEntry()).getDestination().calculateHash();
+        } else {
+            client = msg.getKey();
+        }
 
         Hash to = peer.getIdentity().getHash();
         TunnelInfo replyTunnel = getContext().tunnelManager().selectInboundTunnel(client, to);
@@ -476,7 +499,7 @@ abstract class StoreJob extends JobImpl {
             StoreMessageSelector selector = new StoreMessageSelector(getContext(), getJobId(), peer, token, expiration);
     
             if (_log.shouldLog(Log.DEBUG)) {
-                _log.debug("sending encrypted store to " + peer.getIdentity().getHash() + " through " + outTunnel + ": " + sent);
+                _log.debug(getJobId() + ": sending encrypted store to " + peer.getIdentity().getHash() + " through " + outTunnel + ": " + sent);
             }
             getContext().messageRegistry().registerPending(selector, onReply, onFail);
             getContext().tunnelDispatcher().dispatchOutbound(sent, outTunnel.getSendTunnelId(0), null, to);
@@ -494,6 +517,76 @@ abstract class StoreJob extends JobImpl {
         }
     }
     
+    /**
+     * Send a leaseset store message out an exploratory tunnel,
+     * with the reply to come back through a exploratory tunnel.
+     * Stores are garlic encrypted to hide the identity from the OBEP.
+     *
+     * Only for Meta LS2, for now.
+     *
+     * @param msg must contain a leaseset
+     * @since 0.9.41
+     */
+    private void sendWrappedStoreThroughExploratory(DatabaseStoreMessage msg, RouterInfo peer, long expiration) {
+        long token = 1 + getContext().random().nextLong(I2NPMessage.MAX_ID_VALUE);
+        Hash to = peer.getIdentity().getHash();
+        TunnelInfo replyTunnel = getContext().tunnelManager().selectInboundExploratoryTunnel(to);
+        if (replyTunnel == null) {
+            if (_log.shouldLog(Log.WARN))
+                _log.warn("No inbound expl. tunnels for reply - delaying...");
+            // continueSending() above did an addPending() so remove it here.
+            // This means we will skip the peer next time, can't be helped for now
+            // without modding StoreState
+            _state.replyTimeout(to);
+            Job waiter = new WaitJob(getContext());
+            waiter.getTiming().setStartAfter(getContext().clock().now() + 3*1000);
+            getContext().jobQueue().addJob(waiter);
+            return;
+        }
+        TunnelId replyTunnelId = replyTunnel.getReceiveTunnelId(0);
+        msg.setReplyToken(token);
+        msg.setReplyTunnel(replyTunnelId);
+        msg.setReplyGateway(replyTunnel.getPeer(0));
+
+        if (_log.shouldLog(Log.DEBUG))
+            _log.debug(getJobId() + ": send(dbStore) w/ token expected " + token);
+
+        TunnelInfo outTunnel = getContext().tunnelManager().selectOutboundExploratoryTunnel(to);
+        if (outTunnel != null) {
+            I2NPMessage sent;
+            // garlic encrypt using router SKM
+            MessageWrapper.WrappedMessage wm = MessageWrapper.wrap(getContext(), msg, null, peer);
+            if (wm == null) {
+                if (_log.shouldLog(Log.WARN))
+                    _log.warn("Fail garlic encrypting");
+                fail();
+                return;
+            }
+            sent = wm.getMessage();
+            _state.addPending(to, wm);
+
+            SendSuccessJob onReply = new SendSuccessJob(getContext(), peer, outTunnel, sent.getMessageSize());
+            FailedJob onFail = new FailedJob(getContext(), peer, getContext().clock().now());
+            StoreMessageSelector selector = new StoreMessageSelector(getContext(), getJobId(), peer, token, expiration);
+    
+            if (_log.shouldLog(Log.DEBUG)) {
+                _log.debug(getJobId() + ": sending encrypted store to " + peer.getIdentity().getHash() + " through " + outTunnel + ": " + sent);
+            }
+            getContext().messageRegistry().registerPending(selector, onReply, onFail);
+            getContext().tunnelDispatcher().dispatchOutbound(sent, outTunnel.getSendTunnelId(0), null, to);
+        } else {
+            if (_log.shouldLog(Log.WARN))
+                _log.warn("No outbound expl. tunnels to send a dbStore out - delaying...");
+            // continueSending() above did an addPending() so remove it here.
+            // This means we will skip the peer next time, can't be helped for now
+            // without modding StoreState
+            _state.replyTimeout(to);
+            Job waiter = new WaitJob(getContext());
+            waiter.getTiming().setStartAfter(getContext().clock().now() + 3*1000);
+            getContext().jobQueue().addJob(waiter);
+        }
+    }
+    
     /**
      * Called to wait a little while
      * @since 0.7.10
@@ -512,7 +605,7 @@ abstract class StoreJob extends JobImpl {
     public static final String MIN_STORE_VERSION = "0.9.28";
 
     /**
-     * Is it too old?
+     * Is it new enough?
      * @since 0.9.33
      */
     static boolean shouldStoreTo(RouterInfo ri) {
@@ -524,7 +617,7 @@ abstract class StoreJob extends JobImpl {
     public static final String MIN_STORE_LS2_VERSION = "0.9.38";
 
     /**
-     * Is it too old?
+     * Is it new enough?
      * @since 0.9.38
      */
     static boolean shouldStoreLS2To(RouterInfo ri) {
@@ -532,6 +625,21 @@ abstract class StoreJob extends JobImpl {
         return VersionComparator.comp(v, MIN_STORE_LS2_VERSION) >= 0;
     }
 
+    /**
+     * Was supported in 38, but they're now sigtype 11 which wasn't added until 39
+     * @since 0.9.39
+     */
+    public static final String MIN_STORE_ENCLS2_VERSION = "0.9.39";
+
+    /**
+     * Is it new enough?
+     * @since 0.9.39
+     */
+    static boolean shouldStoreEncLS2To(RouterInfo ri) {
+        String v = ri.getVersion();
+        return VersionComparator.comp(v, MIN_STORE_ENCLS2_VERSION) >= 0;
+    }
+
     /**
      * Called after sending a dbStore to a peer successfully, 
      * marking the store as successful
@@ -575,7 +683,7 @@ abstract class StoreJob extends JobImpl {
 
             if ( (_sendThrough != null) && (_msgSize > 0) ) {
                 if (_log.shouldDebug())
-                    _log.debug("sent a " + _msgSize + " byte netDb message through tunnel " + _sendThrough + " after " + howLong);
+                    _log.debug(StoreJob.this.getJobId() + ": sent a " + _msgSize + " byte netDb message through tunnel " + _sendThrough + " after " + howLong);
                 for (int i = 0; i < _sendThrough.getLength(); i++)
                     getContext().profileManager().tunnelDataPushed(_sendThrough.getPeer(i), howLong, _msgSize);
                 _sendThrough.incrementVerifiedBytesTransferred(_msgSize);
@@ -634,10 +742,11 @@ abstract class StoreJob extends JobImpl {
      * Send was totally successful
      */
     protected void succeed() {
-        if (_log.shouldLog(Log.INFO))
+        if (_log.shouldInfo()) {
             _log.info(getJobId() + ": Succeeded sending key " + _state.getTarget());
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug(getJobId() + ": State of successful send: " + _state);
+            if (_log.shouldDebug())
+                _log.debug(getJobId() + ": State of successful send: " + _state);
+        }
         if (_onSuccess != null)
             getContext().jobQueue().addJob(_onSuccess);
         _state.complete(true);
@@ -648,10 +757,11 @@ abstract class StoreJob extends JobImpl {
      * Send totally failed
      */
     protected void fail() {
-        if (_log.shouldLog(Log.INFO))
+        if (_log.shouldInfo()) {
             _log.info(getJobId() + ": Failed sending key " + _state.getTarget());
-        if (_log.shouldLog(Log.DEBUG))
-            _log.debug(getJobId() + ": State of failed send: " + _state, new Exception("Who failed me?"));
+            if (_log.shouldDebug())
+                _log.debug(getJobId() + ": State of failed send: " + _state, new Exception("Who failed me?"));
+        }
         if (_onFailure != null)
             getContext().jobQueue().addJob(_onFailure);
         _state.complete(true);
