@@ -24,8 +24,11 @@ import com.maxmind.geoip2.DatabaseReader;
 import net.i2p.I2PAppContext;
 import net.i2p.data.DataHelper;
 import net.i2p.data.Hash;
+import net.i2p.data.router.RouterAddress;
+import net.i2p.data.router.RouterInfo;
 import net.i2p.router.Router;
 import net.i2p.router.RouterContext;
+import net.i2p.router.transport.udp.UDPTransport;
 import net.i2p.util.Addresses;
 import net.i2p.util.ConcurrentHashSet;
 import net.i2p.util.Log;
@@ -125,6 +128,8 @@ public class GeoIP {
     /**
      * Blocking lookup of all pending IPs.
      * Results will be added to the table and available via get() after completion.
+     *
+     * Public for BundleRouterInfos
      */
     public void blockingLookup() {
         if (! _context.getBooleanPropertyDefaultTrue(PROP_GEOIP_ENABLED)) {
@@ -153,10 +158,19 @@ public class GeoIP {
                 return;
             File geoip2 = getGeoIP2();
             DatabaseReader dbr = null;
+            long start = _context.clock().now();
             try {
                 // clear the negative cache every few runs, to prevent it from getting too big
                 if (((++_lookupRunCount) % CLEAR) == 0)
                     _notFound.clear();
+                // add our detected addresses
+                Set<String> addrs = Addresses.getAddresses(false, true);
+                for (String ip : addrs) {
+                    add(ip);
+                }
+                String lastIP = _context.getProperty(UDPTransport.PROP_IP);
+                if (lastIP != null)
+                    add(lastIP);
                 // IPv4
                 Long[] search = _pendingSearch.toArray(new Long[_pendingSearch.size()]);
                 _pendingSearch.clear();
@@ -295,6 +309,8 @@ public class GeoIP {
                 if (dbr != null) try { dbr.close(); } catch (IOException ioe) {}
                 _lock.set(false);
             }
+            if (_log.shouldInfo())
+                _log.info("GeoIP processing finished, time: " + (_context.clock().now() - start));
         }
     }
 
@@ -423,7 +439,6 @@ public class GeoIP {
         }
         String[] rv = new String[search.length];
         int idx = 0;
-        long start = _context.clock().now();
         BufferedReader br = null;
         try {
             String buf = null;
@@ -459,9 +474,6 @@ public class GeoIP {
             if (br != null) try { br.close(); } catch (IOException ioe) {}
         }
 
-        if (_log.shouldLog(Log.INFO)) {
-            _log.info("GeoIP processing finished, time: " + (_context.clock().now() - start));
-        }
         return rv;
     }
 
@@ -476,27 +488,59 @@ public class GeoIP {
             return;
         RouterContext ctx = (RouterContext) _context;
         String oldCountry = ctx.router().getConfigSetting(PROP_IP_COUNTRY);
-        Hash ourHash = ctx.routerHash();
+        RouterInfo us = ctx.router().getRouterInfo();
+        String country = null;
         // we should always have a RouterInfo by now, but we had one report of an NPE here
-        if (ourHash == null)
-            return;
-        String country = ctx.commSystem().getCountry(ourHash);
+        if (us != null) {
+            // try our published addresses
+            for (RouterAddress ra : us.getAddresses()) {
+                byte[] ip = ra.getIP();
+                if (ip != null) {
+                    country = get(ip);
+                    if (country != null)
+                        break;
+                }
+            }
+        }
+        if (country == null) {
+            // try our detected addresses
+            Set<String> addrs = Addresses.getAddresses(false, true);
+            for (String ip : addrs) {
+                country = get(ip);
+                if (country != null)
+                    break;
+            }
+            if (country == null) {
+                String lastIP = _context.getProperty(UDPTransport.PROP_IP);
+                if (lastIP != null)
+                    country = get(lastIP);
+            }
+        }
+        if (_log.shouldInfo())
+            _log.info("Old country was " + oldCountry + " new country is " + country);
         if (country != null && !country.equals(oldCountry)) {
+            boolean wasStrict = ctx.commSystem().isInStrictCountry();
             ctx.router().saveConfig(PROP_IP_COUNTRY, country);
-            if (ctx.commSystem().isInBadCountry() && ctx.getProperty(Router.PROP_HIDDEN_HIDDEN) == null) {
-                String name = fullName(country);
-                if (name == null)
-                    name = country;
-                _log.logAlways(Log.WARN, "Setting hidden mode to protect you in " + name +
-                                         ", you may override on the network configuration page");
+            boolean isStrict = ctx.commSystem().isInStrictCountry();
+            if (_log.shouldInfo())
+                _log.info("Old country was strict? " + wasStrict + "; new country is strict? " + isStrict);
+            if (wasStrict != isStrict && ctx.getProperty(Router.PROP_HIDDEN_HIDDEN) == null) {
+                if (isStrict) {
+                    String name = fullName(country);
+                    if (name == null)
+                        name = country;
+                    _log.logAlways(Log.WARN, "Setting hidden mode to protect you in " + name +
+                                             ", you may override on the network configuration page");
+                }
                 ctx.router().rebuildRouterInfo();
             }
         }
-        /****/
     }
 
     /**
      * Add to the list needing lookup
+     * Public for BundleRouterInfos
+     *
      * @param ip IPv4 or IPv6
      */
     public void add(String ip) {
@@ -507,6 +551,8 @@ public class GeoIP {
 
     /**
      * Add to the list needing lookup
+     * Public for BundleRouterInfos
+     *
      * @param ip IPv4 or IPv6
      */
     public void add(byte ip[]) {
@@ -526,6 +572,8 @@ public class GeoIP {
 
     /**
      * Get the country for an IP from the cache.
+     * Public for BundleRouterInfos
+     *
      * @param ip IPv4 or IPv6
      * @return lower-case code, generally two letters, or null.
      */
@@ -540,7 +588,7 @@ public class GeoIP {
      * @param ip IPv4 or IPv6
      * @return lower-case code, generally two letters, or null.
      */
-    public String get(byte ip[]) {
+    String get(byte ip[]) {
         return get(toLong(ip));
     }
 
@@ -594,6 +642,8 @@ public class GeoIP {
 
     /**
      * Get the country for a country code
+     * Public for BundleRouterInfos
+     *
      * @param code two-letter lower case code
      * @return untranslated name or null
      */
