@@ -42,6 +42,8 @@ import net.i2p.router.transport.ntcp.NTCPTransport;
 import net.i2p.router.transport.udp.UDPTransport;
 import net.i2p.util.Addresses;
 import net.i2p.util.Log;
+import net.i2p.util.SimpleTimer;
+import net.i2p.util.SimpleTimer2;
 import net.i2p.util.SystemVersion;
 import net.i2p.util.Translate;
 import net.i2p.util.VersionComparator;
@@ -70,6 +72,9 @@ public class TransportManager implements TransportEventListener {
     private final UPnPManager _upnpManager;
     private final DHSessionKeyBuilder.PrecalcRunner _dhThread;
     private final X25519KeyFactory _xdhThread;
+    private final boolean _enableUDP;
+    private final boolean _enableNTCP1;
+    private boolean _upnpUpdateQueued;
 
     /** default true */
     public final static String PROP_ENABLE_UDP = "i2np.udp.enable";
@@ -78,6 +83,9 @@ public class TransportManager implements TransportEventListener {
     /** default true */
     public final static String PROP_ENABLE_UPNP = "i2np.upnp.enable";
 
+    /** default true */
+    private static final String PROP_NTCP1_ENABLE = "i2np.ntcp1.enable";
+    private static final boolean DEFAULT_NTCP1_ENABLE = false;
     private static final String PROP_NTCP2_ENABLE = "i2np.ntcp2.enable";
     private static final boolean DEFAULT_NTCP2_ENABLE = true;
 
@@ -102,9 +110,12 @@ public class TransportManager implements TransportEventListener {
             _upnpManager = new UPnPManager(context, this);
         else
             _upnpManager = null;
-        _dhThread = new DHSessionKeyBuilder.PrecalcRunner(context);
+        _enableUDP = _context.getBooleanPropertyDefaultTrue(PROP_ENABLE_UDP);
+        _enableNTCP1 = isNTCPEnabled(context) &&
+                       context.getProperty(PROP_NTCP1_ENABLE, DEFAULT_NTCP1_ENABLE);
         boolean enableNTCP2 = isNTCPEnabled(context) &&
                               context.getProperty(PROP_NTCP2_ENABLE, DEFAULT_NTCP2_ENABLE);
+        _dhThread = (_enableUDP || enableNTCP2) ? new DHSessionKeyBuilder.PrecalcRunner(context) : null;
         _xdhThread = enableNTCP2 ? new X25519KeyFactory(context) : null;
     }
 
@@ -149,6 +160,7 @@ public class TransportManager implements TransportEventListener {
     /**
      *  Hook for pluggable transport creation.
      *
+     *  @return null if both NTCP1 and SSU are disabled
      *  @since 0.9.16
      */
     DHSessionKeyBuilder.Factory getDHFactory() {
@@ -172,15 +184,15 @@ public class TransportManager implements TransportEventListener {
     }
 
     private void configTransports() {
-        boolean enableUDP = _context.getBooleanPropertyDefaultTrue(PROP_ENABLE_UDP);
         Transport udp = null;
-        if (enableUDP) {
+        if (_enableUDP) {
             udp = new UDPTransport(_context, _dhThread);
             addTransport(udp);
             initializeAddress(udp);
         }
         if (isNTCPEnabled(_context)) {
-            Transport ntcp = new NTCPTransport(_context, _dhThread, _xdhThread);
+            DHSessionKeyBuilder.PrecalcRunner dh = _enableNTCP1 ? _dhThread : null;
+            Transport ntcp = new NTCPTransport(_context, dh, _xdhThread);
             addTransport(ntcp);
             initializeAddress(ntcp);
             if (udp != null) {
@@ -189,6 +201,11 @@ public class TransportManager implements TransportEventListener {
                 int port = udp.getRequestedPort();
                 if (port > 0)
                     ntcp.externalAddressReceived(SOURCE_CONFIG, (byte[]) null, port);
+            } else {
+                // SSU disabled
+                int port = ntcp.getRequestedPort();
+                if (port > 0)
+                    ntcp.externalAddressReceived(SOURCE_CONFIG, (byte[]) null, port);
             }
         }
         if (_transports.isEmpty())
@@ -315,7 +332,7 @@ public class TransportManager implements TransportEventListener {
     }
 
     synchronized void startListening() {
-        if (_dhThread.getState() == Thread.State.NEW)
+        if (_dhThread != null && _dhThread.getState() == Thread.State.NEW)
             _dhThread.start();
         if (_xdhThread != null && _xdhThread.getState() == Thread.State.NEW)
             _xdhThread.start();
@@ -324,7 +341,10 @@ public class TransportManager implements TransportEventListener {
         // Maybe we need a config option to force on? Probably not.
         // What firewall supports UPnP and is configured with a public address on the LAN side?
         // Unlikely.
-        if (_upnpManager != null && Addresses.getAnyAddress() == null)
+        // Always start on Android, as we may have a cellular IPv4 address but
+        // are routing all traffic through WiFi.
+        // Also, conditions may change rapidly.
+        if (_upnpManager != null && (SystemVersion.isAndroid() || Addresses.getAnyAddress() == null))
             _upnpManager.start();
         configTransports();
         _log.debug("Starting up the transport manager");
@@ -377,7 +397,8 @@ public class TransportManager implements TransportEventListener {
      */
     synchronized void shutdown() {
         stopListening();
-        _dhThread.shutdown();
+        if (_dhThread != null)
+            _dhThread.shutdown();
         if (_xdhThread != null)
             _xdhThread.shutdown();
         Addresses.clearCaches();
@@ -651,7 +672,7 @@ public class TransportManager implements TransportEventListener {
                 _context.getBooleanProperty(NTCPTransport.PROP_I2NP_NTCP_AUTO_PORT)) {
                 Transport udp = getTransport(UDPTransport.STYLE);
                 if (udp != null)
-                    port = t.getRequestedPort();
+                    port = udp.getRequestedPort();
             }
             if (port > 0)
                 rv.add(new Port(t.getStyle(), port));
@@ -796,9 +817,32 @@ public class TransportManager implements TransportEventListener {
      */
     public void transportAddressChanged() {
         if (_upnpManager != null) {
-            _upnpManager.rescan();
-            // should really delay the following by 5 seconds?
-            _upnpManager.update(getPorts());
+            synchronized (_upnpManager) {
+                if (!_upnpUpdateQueued) {
+                    boolean shouldWait = _upnpManager.rescan();
+                    if (shouldWait) {
+                        // Delay until the rescan finishes, MX time + 250
+                        _upnpUpdateQueued = true;
+                        _context.simpleTimer2().addEvent(new UpdatePorts(), 3250);
+                    } else {
+                        _upnpManager.update(getPorts());
+                    }
+                }
+            }
+        }
+    }
+
+    /**
+     * Delayed update of UPnP ports
+     *
+     * @since 0.9.39
+     */
+    private class UpdatePorts implements SimpleTimer.TimedEvent {
+        public void timeReached() {
+            synchronized (_upnpManager) {
+                _upnpUpdateQueued = false;
+                _upnpManager.update(getPorts());
+            }
         }
     }
 
