@@ -47,6 +47,7 @@ import net.i2p.router.transport.Transport;
 import static net.i2p.router.transport.Transport.AddressSource.*;
 import net.i2p.router.transport.TransportBid;
 import net.i2p.router.transport.TransportImpl;
+import net.i2p.router.transport.TransportManager;
 import net.i2p.router.transport.TransportUtil;
 import static net.i2p.router.transport.TransportUtil.IPv6Config.*;
 import net.i2p.router.transport.crypto.DHSessionKeyBuilder;
@@ -137,6 +138,7 @@ public class NTCPTransport extends TransportImpl {
     private static final int NTCP2_IV_LEN = OutboundNTCP2State.IV_SIZE;
     private static final int NTCP2_KEY_LEN = OutboundNTCP2State.KEY_SIZE;
     private static final long MIN_DOWNTIME_TO_REKEY = 30*24*60*60*1000L;
+    private final boolean _enableNTCP1;
     private final boolean _enableNTCP2;
     private final byte[] _ntcp2StaticPubkey;
     private final byte[] _ntcp2StaticPrivkey;
@@ -145,6 +147,7 @@ public class NTCPTransport extends TransportImpl {
     private final String _b64Ntcp2StaticIV;
 
     /**
+     *  @param dh null to disable NTCP1
      *  @param xdh null to disable NTCP2
      */
     public NTCPTransport(RouterContext ctx, DHSessionKeyBuilder.Factory dh, X25519KeyFactory xdh) {
@@ -238,7 +241,11 @@ public class NTCPTransport extends TransportImpl {
         _nearCapacityCostBid = new SharedBid(105);
         _transientFail = new SharedBid(TransportBid.TRANSIENT_FAIL);
 
+        setupPort();
+        _enableNTCP1 = dh != null;
         _enableNTCP2 = xdh != null;
+        if (!_enableNTCP1 && !_enableNTCP2)
+            throw new IllegalArgumentException();
         if (_enableNTCP2) {
             boolean shouldSave = false;
             byte[] priv = null;
@@ -300,6 +307,28 @@ public class NTCPTransport extends TransportImpl {
             _b64Ntcp2StaticIV = null;
         }
     }
+    
+    /**
+     *  Pick a port if not previously configured.
+     *  Only if UDP is disabled.
+     *
+     *  @since 0.9.39
+     */
+    private void setupPort() {
+        if (_context.getBooleanPropertyDefaultTrue(TransportManager.PROP_ENABLE_UDP))
+            return;
+        int port = getRequestedPort();
+        if (port > 0 && !TransportUtil.isValidPort(port)) {
+            TransportUtil.logInvalidPort(_log, STYLE, port);
+        }
+        if (port <= 0) {
+            port = TransportUtil.selectRandomPort(_context, STYLE);
+            Map<String, String> changes = new HashMap<String, String>(2);
+            changes.put(PROP_I2NP_NTCP_PORT, Integer.toString(port));
+            _context.router().saveConfig(changes, null);
+            _log.logAlways(Log.INFO, "NTCP selected random port " + port);
+        }
+    }
 
     /**
      * @param con that is established
@@ -577,6 +606,10 @@ public class NTCPTransport extends TransportImpl {
         List<RouterAddress> addrs = getTargetAddresses(target);
         for (int i = 0; i < addrs.size(); i++) {
             RouterAddress addr = addrs.get(i);
+            // use this to skip outbound-only NTCP2,
+            // and NTCP1 if disabled
+            if (getNTCPVersion(addr) == 0)
+                continue;
             byte[] ip = addr.getIP();
             if (!TransportUtil.isValidPort(addr.getPort()) || ip == null) {
                 //_context.statManager().addRateData("ntcp.connectFailedInvalidPort", 1);
@@ -614,7 +647,7 @@ public class NTCPTransport extends TransportImpl {
     }
 
     public boolean allowConnection() {
-        return countActivePeers() < getMaxConnections();
+        return _conByIdent.size() < getMaxConnections();
     }
 
     /** queue up afterSend call, which can take some time w/ jobs, etc */
@@ -840,7 +873,7 @@ public class NTCPTransport extends TransportImpl {
                     props.setProperty(RouterAddress.PROP_PORT, Integer.toString(port));
                     addNTCP2Options(props);
                     int cost = getDefaultCost(ia instanceof Inet6Address);
-                    myAddress = new RouterAddress(STYLE, props, cost);
+                    myAddress = new RouterAddress(getPublishStyle(), props, cost);
                     replaceAddress(myAddress);
                 }
             } else if (_enableNTCP2) {
@@ -969,7 +1002,7 @@ public class NTCPTransport extends TransportImpl {
                     props.setProperty(RouterAddress.PROP_PORT, Integer.toString(port));
                     addNTCP2Options(props);
                     int cost = getDefaultCost(false);
-                    myAddress = new RouterAddress(STYLE, props, cost);
+                    myAddress = new RouterAddress(getPublishStyle(), props, cost);
                 }
                 if (!_endpoints.isEmpty()) {
                     // If we are already bound to the new address, OR
@@ -990,6 +1023,7 @@ public class NTCPTransport extends TransportImpl {
                     _log.error("Specified NTCP port is " + port + ", ports lower than 1024 not recommended");
                 ServerSocketChannel chan = ServerSocketChannel.open();
                 chan.configureBlocking(false);
+                // TODO retry
                 chan.socket().bind(addr);
                 _endpoints.add(addr);
                 if (_log.shouldLog(Log.INFO))
@@ -1052,6 +1086,9 @@ public class NTCPTransport extends TransportImpl {
      */
     net.i2p.router.transport.ntcp.Writer getWriter() { return _writer; }
 
+    /**
+     * @return always "NTCP" even if NTCP1 is disabled
+     */
     public String getStyle() { return STYLE; }
 
     /**
@@ -1064,16 +1101,25 @@ public class NTCPTransport extends TransportImpl {
         return _enableNTCP2 ? STYLE2 : null;
     }
 
+    /**
+     * @return "NTCP" if NTCP1 is enabled, else "NTCP2"
+     * @since 0.9.39
+     */
+    private String getPublishStyle() {
+        return _enableNTCP1 ? STYLE : STYLE2;
+    }
+
     /**
      *  Hook for NTCPConnection
      */
     EventPumper getPumper() { return _pumper; }
 
     /**
+     *  @return null if not configured for NTCP1
      *  @since 0.9
      */
     DHSessionKeyBuilder getDHBuilder() {
-        return _dhFactory.getBuilder();
+        return _dhFactory != null ? _dhFactory.getBuilder() : null;
     }
 
     /**
@@ -1092,7 +1138,8 @@ public class NTCPTransport extends TransportImpl {
      * @since 0.9.16
      */
     void returnUnused(DHSessionKeyBuilder builder) {
-        _dhFactory.returnUnused(builder);
+        if (_dhFactory != null)
+            _dhFactory.returnUnused(builder);
     }
 
     /**
@@ -1185,7 +1232,7 @@ public class NTCPTransport extends TransportImpl {
         props.setProperty(RouterAddress.PROP_PORT, Integer.toString(p));
         addNTCP2Options(props);
         int cost = getDefaultCost(false);
-        RouterAddress addr = new RouterAddress(STYLE, props, cost);
+        RouterAddress addr = new RouterAddress(getPublishStyle(), props, cost);
         return addr;
     }
 
@@ -1205,6 +1252,13 @@ public class NTCPTransport extends TransportImpl {
         props.setProperty("v", NTCP2_VERSION);
     }
 
+    /**
+     * Is NTCP1 enabled?
+     *
+     * @since 0.9.39
+     */
+    boolean isNTCP1Enabled() { return _enableNTCP1; }
+
     /**
      * Is NTCP2 enabled?
      *
@@ -1267,9 +1321,11 @@ public class NTCPTransport extends TransportImpl {
             addr.getOption("i") == null ||
             addr.getOption("s") == null ||
             (!v.equals(NTCP2_VERSION) && !v.startsWith(NTCP2_VERSION_ALT))) {
-            return (rv == 1) ? 1 : 0;
+            // his address is NTCP1 or is outbound NTCP2 only
+            return (rv == 1 && _enableNTCP1) ? 1 : 0;
         }
-        // todo validate s/i b64, or just catch it later?
+        // his address is NTCP2
+        // do not validate the s/i b64, we will just catch it later
         return NTCP2_INT_VERSION;
     }
 
@@ -1406,11 +1462,16 @@ public class NTCPTransport extends TransportImpl {
             }
             return;
         }
-        // ignore UPnP for now, get everything from SSU
-        if (source != SOURCE_SSU)
+        // ignore UPnP for now, get everything from SSU if it's enabled
+        boolean ssuEnabled = _context.getBooleanPropertyDefaultTrue(TransportManager.PROP_ENABLE_UDP);
+        if (source != SOURCE_SSU && ssuEnabled)
             return;
+        Status old = ssuEnabled ? null : getReachabilityStatus();
         boolean isIPv6 = ip != null && ip.length == 16;
-        externalAddressReceived(ip, isIPv6, port);
+        boolean changed = externalAddressReceived(ip, isIPv6, port);
+        if (changed && !ssuEnabled) {
+            addressChanged(old);
+        }
     }
 
     /**
@@ -1433,20 +1494,45 @@ public class NTCPTransport extends TransportImpl {
     public void externalAddressRemoved(AddressSource source, boolean ipv6) {
         if (_log.shouldWarn())
             _log.warn("Removing address, ipv6? " + ipv6 + " from: " + source, new Exception());
-        // ignore UPnP for now, get everything from SSU
-        if (source != SOURCE_SSU)
+        // ignore UPnP for now, get everything from SSU if it's enabled
+        boolean ssuEnabled = _context.getBooleanPropertyDefaultTrue(TransportManager.PROP_ENABLE_UDP);
+        if (source != SOURCE_SSU && ssuEnabled)
             return;
-        externalAddressReceived(null, ipv6, 0);
+        Status old = ssuEnabled ? null : getReachabilityStatus();
+        boolean changed = externalAddressReceived(null, ipv6, 0);
+        if (changed && !ssuEnabled) {
+            addressChanged(old);
+        }
     }    
+
+    /**
+     *  Only called if SSU is disabled AND our address changed.
+     *  Tell the event log, and tell the router.
+     *
+     *  @since 0.9.40
+     */
+    private void addressChanged(Status old) {
+        Status status = getReachabilityStatus();
+        if (status != old) {
+            if (_log.shouldLog(Log.WARN))
+                _log.warn("Old status: " + old + " New status: " + status +
+                          " from: ", new Exception("traceback"));
+            if (old != Status.UNKNOWN)
+                _context.router().eventLog().addEvent(EventLog.REACHABILITY,
+                   "from " + _t(old.toStatusString()) + " to " +  _t(status.toStatusString()));
+        }
+        _context.router().rebuildRouterInfo();
+    }
     
     /**
      *  UDP changed addresses, tell NTCP and restart.
      *  Port may be set to indicate requested port even if ip is null.
      *
      *  @param ip previously validated; may be null to indicate IPv4 failure or port info only
+     *  @return true if our address changed
      *  @since IPv6 moved from CSFI.notifyReplaceAddress()
      */
-    private synchronized void externalAddressReceived(byte[] ip, boolean isIPv6, int port) {
+    private synchronized boolean externalAddressReceived(byte[] ip, boolean isIPv6, int port) {
         // FIXME just take first address for now
         // FIXME if SSU set to hostname, NTCP will be set to IP
         RouterAddress oldAddr = getCurrentAddress(isIPv6);
@@ -1461,7 +1547,7 @@ public class NTCPTransport extends TransportImpl {
             cost = oldAddr.getCost();
             newProps.putAll(oldAddr.getOptionsMap());
         }
-        RouterAddress newAddr = new RouterAddress(STYLE, newProps, cost);
+        RouterAddress newAddr = new RouterAddress(getPublishStyle(), newProps, cost);
 
         boolean changed = false;
 
@@ -1522,16 +1608,18 @@ public class NTCPTransport extends TransportImpl {
             if (!ssuOK) {
                 if (_log.shouldLog(Log.WARN))
                     _log.warn("null address with always config", new Exception());
-                return;
+                return false;
             }
             // ip non-null
             String nhost = Addresses.toString(ip);
             if (_log.shouldLog(Log.INFO))
                 _log.info("old: " + ohost + " config: " + name + " new: " + nhost);
             if (nhost == null || nhost.length() <= 0)
-                return;
+                return false;
             if (ohost == null || ! ohost.equalsIgnoreCase(nhost)) {
                 newProps.setProperty(RouterAddress.PROP_HOST, nhost);
+                if (cost == NTCP2_OUTBOUND_COST)
+                    newAddr.setCost(DEFAULT_COST);
                 changed = true;
             }
         } else if (enabled.equals("false") &&
@@ -1543,9 +1631,11 @@ public class NTCPTransport extends TransportImpl {
             if (_log.shouldLog(Log.INFO))
                 _log.info("old host: " + ohost + " config: " + name + " new: " + name);
             newProps.setProperty(RouterAddress.PROP_HOST, name);
+            if (cost == NTCP2_OUTBOUND_COST)
+                newAddr.setCost(DEFAULT_COST);
             changed = true;
         } else if (ohost == null || ohost.length() <= 0) {
-            return;
+            return false;
         } else if (Boolean.parseBoolean(enabled) && !ssuOK) {
             // UDP transitioned to not-OK, turn off NTCP address
             // This will commonly happen at startup if we were initially OK
@@ -1577,11 +1667,11 @@ public class NTCPTransport extends TransportImpl {
                     // fall thru and republish
                 } else {
                     _log.info("No change to NTCP Address");
-                    return;
+                    return false;
                 }
             } else {
                 _log.info("No change to NTCP Address");
-                return;
+                return false;
             }
         }
         addNTCP2Options(newProps);
@@ -1603,7 +1693,7 @@ public class NTCPTransport extends TransportImpl {
         restartListening(newAddr, isIPv6);
         if (_log.shouldLog(Log.WARN))
             _log.warn("Updating NTCP Address (ipv6? " + isIPv6 + ") with " + newAddr);
-        return;     	
+        return true;     	
     }
 
     /**
@@ -1624,6 +1714,13 @@ public class NTCPTransport extends TransportImpl {
             else
                 _log.warn("UPnP has failed to open the NTCP port: " + port + " reason: " + reason);
         }
+        // if SSU is disabled, externalAddressReceived() will update our address and call rebuildRouterInfo().
+        // getReachabilityStatus() should report correctly after address is updated.
+        //if (!_context.getBooleanPropertyDefaultTrue(TransportManager.PROP_ENABLE_UDP)) {
+        //    if (success && ip != null && isValid(ip) && !isIPv4Firewalled()) {
+        //        ...
+        //    }
+        //}
     }
 
     /**
@@ -1674,10 +1771,12 @@ public class NTCPTransport extends TransportImpl {
             v6Disabled = false;
         }
         boolean hasV4 = getCurrentAddress(false) != null;
-        // or use _haveIPv6Addrnss ??
         boolean hasV6 = getCurrentAddress(true) != null;
-        if (!hasV4 && !hasV6)
-            return Status.UNKNOWN;
+        boolean showFirewalled = !_context.getBooleanPropertyDefaultTrue(TransportManager.PROP_ENABLE_UDP) &&
+                                 _context.router().getUptime() > 10*60*1000;
+        if (!hasV4 && !hasV6) {
+            return showFirewalled ? Status.REJECT_UNSOLICITED : Status.UNKNOWN;
+        }
         long now = _context.clock().now();
         boolean v4OK = hasV4 && !v4Disabled && now - _lastInboundIPv4 < 10*60*1000;
         boolean v6OK = hasV6 && !v6Disabled && now - _lastInboundIPv6 < 30*60*1000;
@@ -1686,6 +1785,8 @@ public class NTCPTransport extends TransportImpl {
                 return Status.OK;
             if (v6Disabled)
                 return Status.OK;
+            if (!_haveIPv6Address)
+                return Status.OK;
             if (!hasV6)
                 return Status.IPV4_OK_IPV6_UNKNOWN;
         }
@@ -1693,7 +1794,7 @@ public class NTCPTransport extends TransportImpl {
             if (v4Disabled)
                 return Status.IPV4_DISABLED_IPV6_OK;
             if (!hasV4)
-                return Status.IPV4_UNKNOWN_IPV6_OK;
+                return showFirewalled ? Status.IPV4_FIREWALLED_IPV6_OK : Status.IPV4_UNKNOWN_IPV6_OK;
         }
         for (NTCPConnection con : _conByIdent.values()) {
             if (con.isInbound()) {
@@ -1716,19 +1817,22 @@ public class NTCPTransport extends TransportImpl {
                     if (v4Disabled)
                         return Status.IPV4_DISABLED_IPV6_OK;
                     if (!hasV4)
-                        return Status.IPV4_UNKNOWN_IPV6_OK;
+                        return showFirewalled ? Status.IPV4_FIREWALLED_IPV6_OK : Status.IPV4_UNKNOWN_IPV6_OK;
                 }
             }
         }
-        if (v4OK)
+        if (v4OK) {
+            if (!_haveIPv6Address)
+                return Status.OK;
             return Status.IPV4_OK_IPV6_UNKNOWN;
+        }
         if (v6OK)
-            return Status.IPV4_UNKNOWN_IPV6_OK;
+            return showFirewalled ? Status.IPV4_FIREWALLED_IPV6_OK : Status.IPV4_UNKNOWN_IPV6_OK;
         if (v4Disabled)
             return Status.IPV4_DISABLED_IPV6_UNKNOWN;
-        if (v6Disabled)
-            return Status.UNKNOWN;
-        return Status.UNKNOWN;
+        //if (v6Disabled)
+        //    return Status.UNKNOWN;
+        return showFirewalled ? Status.REJECT_UNSOLICITED : Status.UNKNOWN;
     }
 
     /**
