@@ -314,10 +314,10 @@ public class PeerState {
      */
     public static final int MAX_MTU = Math.max(LARGE_MTU, MAX_IPV6_MTU);
     
-    private static final int MIN_RTO = 100 + ACKSender.ACK_FREQUENCY;
-    private static final int INIT_RTO = 3*1000;
-    public static final int INIT_RTT = INIT_RTO / 2;
-    private static final int MAX_RTO = 15*1000;
+    private static final int MIN_RTO = 1000;
+    private static final int INIT_RTO = 1000;
+    private static final int INIT_RTT = 0;
+    private static final int MAX_RTO = 60*1000;
     private static final int CLOCK_SKEW_FUDGE = (ACKSender.ACK_FREQUENCY * 2) / 3;
     
     /**
@@ -336,8 +336,11 @@ public class PeerState {
     private static final long RESEND_ACK_TIMEOUT = 5*60*1000;
 
     
+    /**
+     *  @param rtt from the EstablishState, or 0 if not available
+     */
     public PeerState(RouterContext ctx, UDPTransport transport,
-                     byte[] remoteIP, int remotePort, Hash remotePeer, boolean isInbound) {
+                     byte[] remoteIP, int remotePort, Hash remotePeer, boolean isInbound, int rtt) {
         _context = ctx;
         _log = ctx.logManager().getLog(PeerState.class);
         _transport = transport;
@@ -366,9 +369,14 @@ public class PeerState {
         }
         //_mtuLastChecked = -1;
         _lastACKSend = -1;
+
         _rto = INIT_RTO;
         _rtt = INIT_RTT;
-        _rttDeviation = _rtt;
+        if (rtt > 0)  
+            recalculateTimeouts(rtt);
+        else
+            _rttDeviation = _rtt;
+            
         _inboundMessages = new HashMap<Long, InboundMessageState>(8);
         _outboundMessages = new CachedIteratorCollection<OutboundMessageState>();
         //_outboundQueue = new CoDelPriorityBlockingQueue(ctx, "UDP-PeerState", 32);
@@ -470,7 +478,11 @@ public class PeerState {
      */
     //public boolean getRemoteWantsPreviousACKs() { return _remoteWantsPreviousACKs; }
 
-    /** how many bytes should we send to the peer in a second */
+    /**
+     *  how many bytes should we send to the peer in a second
+     *  1st stat in CWND column, otherwise unused,
+     *  candidate for removal
+     */
     public int getSendWindowBytes() {
         synchronized(_outboundMessages) {
             return _sendWindowBytes;
@@ -674,11 +686,6 @@ public class PeerState {
      */
     //public void remoteDoesNotWantPreviousACKs() { _remoteWantsPreviousACKs = false; }
     
-    /** should we ignore the peer state's congestion window, and let anything through? */
-    private static final boolean IGNORE_CWIN = false;
-    /** should we ignore the congestion window on the first push of every message? */
-    private static final boolean ALWAYS_ALLOW_FIRST_PUSH = false;
-    
     /** 
      * Decrement the remaining bytes in the current period's window,
      * returning true if the full size can be decremented, false if it
@@ -709,8 +716,16 @@ public class PeerState {
             //_sendACKBytes = 0;
             _lastSendRefill = now;
         }
-        //if (true) return true;
-        if (IGNORE_CWIN || size <= _sendWindowBytesRemaining || (ALWAYS_ALLOW_FIRST_PUSH && messagePushCount == 0)) {
+
+        // Ticket 2505
+        // We always send all unacked fragments for a message,
+        // because we don't have any mechanism in OutboundMessageFragments
+        // to track the next send time for fragments individually.
+        // Huge messages that are larger than the window size could never
+        // get sent and block the outbound queue forever.
+        // So we let it through when the window is empty (full window remaining).
+        if (size <= _sendWindowBytesRemaining ||
+            (size > _sendWindowBytes && _sendWindowBytesRemaining >= _sendWindowBytes)) {
             if ( (messagePushCount == 0) && (_concurrentMessagesActive > _concurrentMessagesAllowed) ) {
                 _consecutiveRejections++;
                 _context.statManager().addRateData("udp.rejectConcurrentActive", _concurrentMessagesActive, _consecutiveRejections);
@@ -723,6 +738,8 @@ public class PeerState {
                 _consecutiveRejections = 0;
             }
             _sendWindowBytesRemaining -= size; 
+            if (_sendWindowBytesRemaining < 0)
+                _sendWindowBytesRemaining = 0; 
             _sendBytes += size;
             _lastSendTime = now;
             //if (isForACK) 
@@ -759,20 +776,36 @@ public class PeerState {
     }
   ****/
 
+    /**
+     *  stat in SST column, otherwise unused,
+     *  candidate for removal
+     */
     public int getSlowStartThreshold() { return _slowStartThreshold; }
 
+    /**
+     *  2nd stat in CWND column, otherwise unused,
+     *  candidate for removal
+     */
     public int getConcurrentSends() {
         synchronized(_outboundMessages) {
             return _concurrentMessagesActive;
         }
     }
 
+    /**
+     *  3rd stat in CWND column, otherwise unused,
+     *  candidate for removal
+     */
     public int getConcurrentSendWindow() {
         synchronized(_outboundMessages) {
             return _concurrentMessagesAllowed;
         }
     }
 
+    /**
+     *  4th stat in CWND column, otherwise unused,
+     *  candidate for removal
+     */
     public int getConsecutiveSendRejections() {
         synchronized(_outboundMessages) {
             return _consecutiveRejections;
@@ -1226,16 +1259,15 @@ public class PeerState {
      *  Caller should synch on this
      */
     private void recalculateTimeouts(long lifetime) {
-        // the rttDev calculation matches that recommended in RFC 2988 (beta = 1/4)
-        _rttDeviation = _rttDeviation + (int)(0.25d*(Math.abs(lifetime-_rtt)-_rttDeviation));
-        
-        float scale = RTT_DAMPENING;
-        // the faster we are going, the slower we want to reduce the rtt
-        //if (_sendBps > 0)
-        //    scale = lifetime / ((float)lifetime + (float)_sendBps);
-        //if (scale < 0.001f) scale = 0.001f;
-        
-        _rtt = (int)(_rtt*(1.0f-scale) + (scale)*lifetime);
+        if (_rtt <= 0) {
+            // first measurement
+            _rtt = (int) lifetime;
+            _rttDeviation = _rtt /  2;
+        } else {
+            // the rttDev calculation matches that recommended in RFC 2988 (beta = 1/4)
+            _rttDeviation = (int)((0.75 * _rttDeviation) + (0.25 * Math.abs(lifetime - _rtt)));
+            _rtt = (int)((_rtt * (1.0f - RTT_DAMPENING)) + (RTT_DAMPENING * lifetime));
+        }
         // K = 4
         _rto = Math.min(MAX_RTO, Math.max(minRTO(), _rtt + (_rttDeviation<<2)));
         //if (_log.shouldLog(Log.DEBUG))
@@ -1821,13 +1853,15 @@ public class PeerState {
                               + " remaining"
                               + " for message " + state.getMessageId() + ": " + state);
 
-                if (state.getPushCount() > 0)
+                int rto = getRTO();
+                if (state.getPushCount() > 0) {
                     _retransmitter = state;
+                    rto = Math.min(MAX_RTO, rto << state.getPushCount()); // Section 5.5 RFC 6298
+                }
 
                 if (state.push())
                     _messagesSent++;
             
-                int rto = getRTO();
                 state.setNextSendTime(now + rto);
 
                 //if (peer.getSendWindowBytesRemaining() > 0)
